#version 330
precision highp float;

in vec3 in_Position;
out vec2 uv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};
#define screen vec2(screenX,screenY)

#define EPSILON 1e-1
#define NORMAL_PRECISION 2e-2

vec4 hash(vec4 a) {
	return fract(abs(sin(a.ywxz*766.345)+cos(normalize(a)*4972.92855))*2048.97435+abs(a.wxyz)*.2735);
}

float distf(vec3 p) {
	vec3 sp;
	sp.y = p.y-1.;
	sp.xz = mod(abs(p.xz),4.)-2.;
	return min(p.y,length(sp)-1.);
}

vec3 normf(vec3 p, float bd) {
	return normalize(vec3(distf(p+vec3(NORMAL_PRECISION,0,0))-bd,
						  distf(p+vec3(0,NORMAL_PRECISION,0))-bd,
						  distf(p+vec3(0,0,NORMAL_PRECISION))-bd));
}

void vert() {
	uv = in_Position.xy*vec2(1,screenY/screenX);
	time = float(frame)/float(framerate);
	gl_Position = vec4(in_Position,1);
}
void frag() {
	//initial ray parameters
	vec4 hsh = hash(uv.xyyx*vec4(1,1,.3,.3));
	vec3 rp = vec3(100.,3.+sin(time*.2)*2.,time), orp = rp,
		rd = normalize(vec3(uv+(hsh.xy*2.-1.)/vec2(screenX,screenY),1.));
	rp += rd*hsh.z*1.0;
	
	//raytracing
	for (int i = 0; i < 64; i++) {
		float dst = distf(rp);
		if (dst < EPSILON) break;
		rp += rd*dst;
	}
	
	float bdst = distf(rp),
		rdst = length(rp-orp);
	if (bdst < EPSILON) {
		vec3 nrm = normf(rp,bdst);
		float sv = 0.;
		
		//ambient occlusion
		float asum = 0., as = .1;
		vec3 arp = rp+nrm*.1;
		for (int i = 0; i < 4; i++) {
			float adst = distf(arp);
			asum += (max(0.,as-adst)/as)*max(0.,1.-as);
			arp += nrm*adst;
			as += adst;
		}
		sv += 1.-asum*.3;
		
		sv *= max(0.,1.-rdst/40.);//fog
		
		vec3 col;
		if (rp.y < EPSILON) col = vec3(1.);
		else col = pow(hash(floor(abs(rp.xz)/4.).xyyx*vec4(.1234,.124,.323,.332)).xyz,vec3(2.2));
		gl_FragColor = vec4(col*sv,rdst);
	} else {
		gl_FragColor = vec4(vec3(max(0.,rd.y)),rdst);//background
	}
}
