#version 330
precision highp float;

in vec3 in_Position;
out vec2 uv,suv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};
#define screen vec2(screenX,screenY)

uniform sampler2D Buffer0;

mat2 rot(float a) {
	float sn = sin(a), cs = cos(a);
	return mat2(cs,-sn,sn,cs);
}
vec4 hash(vec4 a) {
	return fract(abs(sin(a.ywxz*766.345)+cos(normalize(a)*4972.92855))*2048.97435+abs(a.wxyz)*.2735);
}
float twave(float v) {
	v = fract(abs(v));
	return v*2.-max(0.,v*4.-2.);
}
float water(vec2 p) {
	float s = 0.;
	for (int i = 3; i < 6; i++) {
		float fi = float(i);
		s += pow(twave((p*rot(fi*3.8263)).x/fi+fi*.673),1.5+fract(fi*2.864)*2.)*fi;
	}
	return s*.8;
}
float clouds(vec3 p) {
	p.xz += time*vec2(.12,.032)+155.;
	p.y -= water(p.xz);
	//p += (sin(p.x*1.9283)*.2+sin(p.y*2.7483)*.14+cos(p.z*2.9834)*.12)*(3.+sin(p.x*.23)+cos(p.z*.1928));
	vec2 fid = floor(p.xz/10.);
	p.y -= 6.;
	p.xz = mod(abs(p.xz),10.)-5.;
	vec4 fsh = hash(fid.xyxy*vec4(189.01828,28.3383,.082823,.01893));
	return (length(max(abs(p)-max(pow(fsh.xyz,vec3(1))*vec3(5,.3,5),0.),0.))-.5);
}
vec4 stars(vec3 p, float s, float a, vec3 c) {
	vec2 ap = p.xz*s/p.y,
	cp = floor(ap)+c.xy;
	return max(hash(cp.xyyx*c.z*vec4(.0638472,.0275841,.7756123,.6789432))/a-1./a+1.-max(0.,length(mod(abs(ap),1.)-.5)-.2)*4.,0.);
}
vec3 hsvToRGB(vec3 hsv) {
	return mix(clamp(abs(mod(hsv.x*6.0+vec3(0.,4.,2.),6.)-3.0)-1.0,0.,1.),vec3(1),hsv.y)*hsv.z;
}

void vert() {
	uv = in_Position.xy*vec2(screenX/screenY,1);
	suv = in_Position.xy*.5+.5;
	time = float(frame)/float(framerate);
	gl_Position = vec4(in_Position,1);
}
void frag() {
	if (frame == 0) {
		gl_FragColor = vec4(0);
		return;
	}
	
	vec4 mhsh = hash(uv.yxyx*vec4(82.98123,78.87812,87.983,90.9812)+float(frame)*vec4(.01,.06231,1.1734,8.87173));
	vec3 ls = vec3(0), cl = vec3(1),
		rp = vec3(0), rd = vec3(normalize(vec3(uv+(mhsh.yz*2.-1.)*vec2(screenX/screenY,1)/vec2(screenX,screenY),1).xzy)), ird = rd;
	float rs = mhsh.x*25.737;
	rp += rd*(4.+fract(rs));
	for (int i = 0; i < 8; i++) {
		float dst = max(.5,clouds(rp));
		if (dst < fract(rs)) {
			//if (fract(rs*.8) < 0.5) rd = normalize(hash((rp.xyzx+rs)*vec4(.02,.382,1.034,9.9872)).xyz);
			//else rd = moonDir;
			rd = normalize(hash((rp.xyzx+rs)*vec4(.02,.382,1.034,9.9872)).xyz);
			cl *= 0.8;
		}
		rp += rd*dst*(.8+fract(rs*1.1)*.2);
		rs += dst*2.2981823;
	}
	vec3 sky = (stars(rd,screenY,1.,vec2(4832.091872,1.).xxy).xyz+.2)*vec3(.1,.05,.3);
	for (int i = 0; i < 4; i++) {
		float fi = float(i), ni = fi/3.;
		vec4 st = stars(rd,80.+fi*fi*20.,.1,vec2(1382.*(1.+fi),.123+fi*.2723).xxy);
		sky += (.5+.5*st.xyz)*st.w*(sin(vec3(3,2,1)-fi*3.872)*.5+.5);
	}
	//sky += stars(rd,100.,.1);
	//sky += max(0.,1.-length(rd-normalize(vec3(.7,.4,0)))*mix(10.,1.,min(1.,length(rd-ird)*1e3)))*4.;
	ls += cl*sky;//stars(rd,50.);//sky;
	gl_FragColor = mix(texture2D(Buffer0,suv),vec4(ls,1),0.01);
}
