#version 150
precision highp float;

in vec3 in_Position;
out vec2 uv;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};
layout(std140) uniform MidiBlock {
	vec4 midi[64];//x = note id, y = velocity, z = channel/track, w = time in seconds since note pressed
	int midiCount;
};
/*example: NoteKeyToId("c4") or NoteKeyToId("a2-")
function NoteKeyToId(key) {
	var scale = (key.charCodeAt(1)-48)
		letter = {"c":0,"d":2,"e":4,"f":5,"g":7,"a":9,"b":11}[key.charAt(0)],
		minor = (key.length>2&&key.charAt(2))==="-";
	return {"c":0,"d":2,"e":4,"f":5,"g":7,"a":9,"b":11}[key.charAt(0)] + //letter
			(key.charCodeAt(1)-47)*12 + //scale
			(key.length>2&&key.charAt(2)==="-"?1:0);
}
*/

//Beat/note length in seconds, for example TEMPO_TIME(140., 1./4.) for the length of a quarter note.
#define TEMPO_TIME(bpm,note) (60.*4.*note/bpm)


void vert() {
	uv = in_Position.xy*vec2(screenX/screenY,1);
	gl_Position = vec4(in_Position,1);
}
void frag() {
	vec4 c = vec4(0);
	for (int i = 0; i < midiCount; i++) {
		if (midi[i].z == 0.) {//drum
			c += vec4(1,0,1,1)/(1.+midi[i].w*20.);
		} else {//piano
			c += step(abs(uv.x-(midi[i].x-70.)*.2),.2);
		}
	}
	gl_FragColor = c;
}
